#ifndef FLATARRAYALLOCALGOR_H_IRJ004
#define FLATARRAYALLOCALGOR_H_IRJ004

#include "TestAlgor.h"
#include "Particle.h"
#include <boost/timer.hpp>
#include <string>

/**
 * The FlatArrayAllocAlgor was contributed by orz from Allegro.cc.  This
 * algorithm is identical to VectorAlgor, except uses an array of static size.
 * It will simply drop particles if there are too many.
 *
 * This version, as orz wrote it, uses 1.8 megs of RAM for the particle
 * list.  It would probably be possible to reduce that significantly ;).
 *
 * @author orz
 */
class FlatArrayAllocAlgor : public TestAlgor {
public:
  FlatArrayAllocAlgor( ) {}

  virtual ~FlatArrayAllocAlgor() {}

  virtual std::string getName() const {
    return "FlatArray 50000";
  }

  virtual double operator()( int numLoops, float dt ) const {
    enum {MAX_P = 50000};
    Particle *parr = new Particle[MAX_P];
    int num = 0;

    boost::timer timer;

    for ( int c = 0; c < numLoops; ++c ) {
      if (num < MAX_P) new (&parr[num++]) Particle(
        Vector3D( getRandF(), getRandF(), getRandF() ),
        Vector3D( getRandF(), getRandF(), getRandF() ),
        getRandI(),
        getRandF(),
        getRandF() );

      for ( int i = 0; i < num; i++ ) {
        parr[i].update2(dt);
        if (parr[i].isDead()) parr[i--] = parr[--num];
      }
    }

    double ret = timer.elapsed();
    delete[] parr;
    return ret;
  }
};

#endif //FLATARRAYALLOCALGOR_H_IRJ004
